@@ -108,11 +108,7 @@
 # define PATH_MAX       1024
 #endif
 
-#if defined(HAVE_OS2_H)
-# define DIR_SEP        ";"
-#elif defined(__CYGWIN__)
-# define DIR_SEP        ":"
-#elif defined(HAVE_WINDOWS_H)
+#if defined(_WIN32) || defined(HAVE_OS2_H)
 # define DIR_SEP        ";"
 #else
 # define DIR_SEP        ":"
@@ -172,19 +168,19 @@ struct backend
 #define BE_ENTRY(be,func)       sane_##be##_##func
 
 #define PRELOAD_DECL(name)                                                            \
-  extern void *BE_ENTRY(name,init) (SANE_Int *, SANE_Auth_Callback);                  \
-  extern void *BE_ENTRY(name,exit) (void);                  \
-  extern void *BE_ENTRY(name,get_devices) (const SANE_Device ***, SANE_Bool);           \
-  extern void *BE_ENTRY(name,open) (SANE_String_Const, SANE_Handle *);                  \
-  extern void *BE_ENTRY(name,close) (SANE_Handle);                 \
-  extern void *BE_ENTRY(name,get_option_descriptor) (SANE_Handle,  SANE_Int); \
-  extern void *BE_ENTRY(name,control_option) (SANE_Handle, SANE_Int, SANE_Action, void *, SANE_Int *);        \
-  extern void *BE_ENTRY(name,get_parameters) (SANE_Handle, SANE_Parameters *);        \
-  extern void *BE_ENTRY(name,start) (SANE_Handle);                 \
-  extern void *BE_ENTRY(name,read) (SANE_Handle, SANE_Byte *, SANE_Int, SANE_Int *);                  \
-  extern void *BE_ENTRY(name,cancel) (SANE_Handle);                \
-  extern void *BE_ENTRY(name,set_io_mode) (SANE_Handle, SANE_Bool);           \
-  extern void *BE_ENTRY(name,get_select_fd) (SANE_Handle, SANE_Int *);
+  extern SANE_Status BE_ENTRY(name,init) (SANE_Int *, SANE_Auth_Callback);                  \
+  extern void BE_ENTRY(name,exit) (void);                  \
+  extern SANE_Status BE_ENTRY(name,get_devices) (const SANE_Device ***, SANE_Bool);           \
+  extern SANE_Status BE_ENTRY(name,open) (SANE_String_Const, SANE_Handle *);                  \
+  extern void BE_ENTRY(name,close) (SANE_Handle);                 \
+  extern const SANE_Option_Descriptor *BE_ENTRY(name,get_option_descriptor) (SANE_Handle,  SANE_Int); \
+  extern SANE_Status BE_ENTRY(name,control_option) (SANE_Handle, SANE_Int, SANE_Action, void *, SANE_Int *);        \
+  extern SANE_Status BE_ENTRY(name,get_parameters) (SANE_Handle, SANE_Parameters *);        \
+  extern SANE_Status BE_ENTRY(name,start) (SANE_Handle);                 \
+  extern SANE_Status BE_ENTRY(name,read) (SANE_Handle, SANE_Byte *, SANE_Int, SANE_Int *);                  \
+  extern void BE_ENTRY(name,cancel) (SANE_Handle);                \
+  extern SANE_Status BE_ENTRY(name,set_io_mode) (SANE_Handle, SANE_Bool);           \
+  extern SANE_Status BE_ENTRY(name,get_select_fd) (SANE_Handle, SANE_Int *);
 
 #define PRELOAD_DEFN(name)                      \
 {                                               \
@@ -211,7 +207,13 @@ struct backend
 }
 
 #ifndef __BEOS__
+#ifdef ENABLE_PRELOAD
 #include "dll-preload.h"
+#else
+static struct backend preloaded_backends[] = {
+ { 0, 0, 0, 0, 0, 0, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }}
+};
+#endif
 #endif
 
 struct meta_scanner
@@ -428,7 +430,7 @@ load (struct backend *be)
 
   if (path)
     {
-      src_len = strlen (path) + strlen (STRINGIFY (LIBDIR)) + 1 + 1;
+      src_len = strlen (path) + strlen (LIBDIR) + 1 + 1;
       src = malloc (src_len);
       if (!src)
 	{
@@ -436,11 +438,11 @@ load (struct backend *be)
 	  return SANE_STATUS_NO_MEM;
 	}
       orig_src = src;
-      snprintf (src, src_len, "%s:%s", path, STRINGIFY (LIBDIR));
+      snprintf (src, src_len, "%s:%s", path, LIBDIR);
     }
   else
     {
-      src = STRINGIFY (LIBDIR);
+      src = LIBDIR;
       src = strdup (src);
       if (!src)
 	{
