@@ -44,34 +44,38 @@
    This file implements a SANE backend for the Polaroid Digital
    Microscope Camera. */
 
-/* $Id: dmc.c,v 1.3 2003/04/03 16:39:56 hmg Exp $ */
+/* $Id$ */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <limits.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 
-#include "sane/sane.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
-#include "dmc.h"
+#include "../include/_stdint.h"
+
+#include "../include/sane/sane.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 
 #define BACKEND_NAME	dmc
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define DMC_CONFIG_FILE "dmc.conf"
 
+#include "dmc.h"
+
 /* A linked-list of attached devices and handles */
 static DMC_Device *FirstDevice = NULL;
 static DMC_Camera *FirstHandle = NULL;
 static int NumDevices = 0;
+static SANE_Device const **devlist = NULL;
 
 static SANE_String_Const ValidModes[] = { "Full frame", "Viewfinder",
 					  "Raw", "Thumbnail",
@@ -121,7 +125,7 @@ static SANE_Status
 DMCRead(int fd, unsigned int typecode, unsigned int qualifier,
 	SANE_Byte *buf, size_t maxlen, size_t *len)
 {
-    u_int8_t readCmd[10];
+    uint8_t readCmd[10];
     SANE_Status status;
 
     readCmd[0] = 0x28;
@@ -160,7 +164,7 @@ static SANE_Status
 DMCWrite(int fd, unsigned int typecode, unsigned int qualifier,
 	SANE_Byte *buf, size_t maxlen)
 {
-    u_int8_t *writeCmd;
+    uint8_t *writeCmd;
     SANE_Status status;
 
     writeCmd = malloc(maxlen + 10);
@@ -208,16 +212,16 @@ DMCAttach(char const *devname, DMC_Device **devp)
     size_t size;
     char result[INQ_LEN];
 
-    u_int8_t exposureCalculationResults[16];
-    u_int8_t userInterfaceSettings[16];
+    uint8_t exposureCalculationResults[16];
+    uint8_t userInterfaceSettings[16];
 
-    static u_int8_t const inquiry[] =
+    static uint8_t const inquiry[] =
     { 0x12, 0x00, 0x00, 0x00, INQ_LEN, 0x00 };
 
-    static u_int8_t const test_unit_ready[] =
+    static uint8_t const test_unit_ready[] =
     { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
-    static u_int8_t const no_viewfinder[] =
+    static uint8_t const no_viewfinder[] =
     { 0xC6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
     /* If we're already attached, do nothing */
@@ -602,7 +606,7 @@ DMCCancel(DMC_Camera *c)
 static SANE_Status
 DMCSetASA(int fd, unsigned int asa)
 {
-    u_int8_t exposureCalculationResults[16];
+    uint8_t exposureCalculationResults[16];
     SANE_Status status;
     size_t len;
     int i;
@@ -619,7 +623,7 @@ DMCSetASA(int fd, unsigned int asa)
     if (status != SANE_STATUS_GOOD) return status;
     if (len < sizeof(exposureCalculationResults)) return SANE_STATUS_IO_ERROR;
 
-    exposureCalculationResults[13] = (u_int8_t) i - 1;
+    exposureCalculationResults[13] = (uint8_t) i - 1;
 
     return DMCWrite(fd, 0x87, 0x4, exposureCalculationResults,
 		    sizeof(exposureCalculationResults));
@@ -638,7 +642,7 @@ DMCSetASA(int fd, unsigned int asa)
 static SANE_Status
 DMCSetWhiteBalance(int fd, int mode)
 {
-    u_int8_t userInterfaceSettings[16];
+    uint8_t userInterfaceSettings[16];
     SANE_Status status;
     size_t len;
 
@@ -648,7 +652,7 @@ DMCSetWhiteBalance(int fd, int mode)
     if (status != SANE_STATUS_GOOD) return status;
     if (len < sizeof(userInterfaceSettings)) return SANE_STATUS_IO_ERROR;
 
-    userInterfaceSettings[5] = (u_int8_t) mode;
+    userInterfaceSettings[5] = (uint8_t) mode;
 
     return DMCWrite(fd, 0x82, 0x0, userInterfaceSettings,
 		    sizeof(userInterfaceSettings));
@@ -667,7 +671,7 @@ DMCSetWhiteBalance(int fd, int mode)
 static SANE_Status
 DMCSetShutterSpeed(int fd, unsigned int speed)
 {
-    u_int8_t exposureCalculationResults[16];
+    uint8_t exposureCalculationResults[16];
     SANE_Status status;
     size_t len;
 
@@ -810,7 +814,7 @@ sane_init(SANE_Int *version_code, SANE_Auth_Callback authorize)
 
     DBG_INIT();
     if (version_code) {
-	*version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+	*version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
     }
 
     fp = sanei_config_open(DMC_CONFIG_FILE);
@@ -864,6 +868,9 @@ sane_exit(void)
 	free(dev);
 	dev = next;
     }
+  
+    if (devlist)
+      free (devlist);
 }
 
 /**********************************************************************
@@ -879,7 +886,6 @@ sane_exit(void)
 SANE_Status
 sane_get_devices(SANE_Device const ***device_list, SANE_Bool local_only)
 {
-    static SANE_Device const **devlist = 0;
     DMC_Device *dev;
     int i = 0;
 
@@ -1165,13 +1171,13 @@ sane_get_parameters(SANE_Handle handle, SANE_Parameters *params)
 SANE_Status
 sane_start(SANE_Handle handle)
 {
-    static u_int8_t const acquire[] =
+    static uint8_t const acquire[] =
     { 0xC1, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
-    static u_int8_t const viewfinder[] =
+    static uint8_t const viewfinder[] =
     { 0xCB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
-    static u_int8_t const no_viewfinder[] =
+    static uint8_t const no_viewfinder[] =
     { 0xC6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
     DMC_Camera *c = ValidateHandle(handle);
