@@ -1,8 +1,8 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2011-2015 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -42,7 +42,7 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
  */
-# include "../include/sane/config.h"
+#include "../include/sane/config.h"
 
 #include <errno.h>
 #include <string.h>
@@ -87,9 +87,12 @@
 
 /* pixma_sane_options.h generated by
  * scripts/pixma_gen_options.py h < pixma.c > pixma_sane_options.h
- * *and* formatted */
+ */
 #include "pixma_sane_options.h"
 
+#define BUTTON_GROUP_SIZE ( opt_scan_resolution - opt_button_1 + 1 )
+#define BUTTON_GROUP_INDEX(x) ( x - opt_button_1 )
+
 typedef struct pixma_sane_t
 {
   struct pixma_sane_t *next;
@@ -103,6 +106,7 @@ typedef struct pixma_sane_t
   SANE_Status last_read_status;	/* valid if !idle && !scanning */
 
   option_descriptor_t opt[opt_last];
+  char button_option_is_cached[BUTTON_GROUP_SIZE];
   SANE_Range xrange, yrange;
   SANE_Word dpi_list[9];	/* up to 9600 dpi */
   SANE_String_Const mode_list[6];
@@ -128,6 +132,13 @@ static pixma_sane_t *first_scanner = NULL;
 static const SANE_Device **dev_list = NULL;
 static const char* conf_devices[MAX_CONF_DEVICES];
 
+static void mark_all_button_options_cached ( struct pixma_sane_t * ss )
+{
+  int i;
+  for (i = 0; i < (opt__group_5 - opt_button_1); i++ )
+      ss -> button_option_is_cached[i] = 1;
+}
+
 static SANE_Status config_attach_pixma(SANEI_Config * config, const char *devname)
 {
   int i;
@@ -272,16 +283,23 @@ update_button_state (pixma_sane_t * ss, SANE_Int * info)
   switch (ev & ~PIXMA_EV_ACTION_MASK)
     {
     case PIXMA_EV_BUTTON1:
-      b1 = (ev & PIXMA_EV_ACTION_MASK) + 1;
+      b1 = 1;
       break;
     case PIXMA_EV_BUTTON2:
-      b2 = (ev & PIXMA_EV_ACTION_MASK) + 1;
+      b2 = 1;
       break;
     }
+
   if (b1 != OVAL (opt_button_1).w || b2 != OVAL (opt_button_2).w)
+    { 
     *info |= SANE_INFO_RELOAD_OPTIONS;
-  OVAL (opt_button_1).w = b1;
-  OVAL (opt_button_2).w = b2;
+    OVAL (opt_button_1).w = b1;
+    OVAL (opt_button_2).w = b2;
+    OVAL (opt_original).w = GET_EV_ORIGINAL(ev);
+    OVAL (opt_target).w = GET_EV_TARGET(ev);
+    OVAL (opt_scan_resolution).w = GET_EV_DPI(ev);
+    }
+  mark_all_button_options_cached(ss);
 }
 
 static SANE_Bool
@@ -335,14 +353,16 @@ clamp_value (pixma_sane_t * ss, SANE_Int n, void *v, SANE_Int * info)
  * tpu = 1: TPU mode
  *          16 bit grayscale and 48 bit color scans */
 static void
-create_mode_list (pixma_sane_t * ss, SANE_Bool tpu)
+create_mode_list (pixma_sane_t * ss)
 {
+  SANE_Bool tpu;
   const pixma_config_t *cfg;
   int i;
 
   cfg = pixma_get_config (ss->s);
+  tpu = (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_TPU);
 
-  /* setup available mode. */
+  /* setup available mode */
   i = 0;
   ss->mode_list[i] = SANE_VALUE_SCAN_MODE_COLOR;
   ss->mode_map[i] = PIXMA_SCAN_MODE_COLOR;
@@ -353,7 +373,7 @@ create_mode_list (pixma_sane_t * ss, SANE_Bool tpu)
       ss->mode_map[i] = PIXMA_SCAN_MODE_GRAY;
       i++;
     }
-  if (tpu && cfg->cap & PIXMA_CAP_NEGATIVE)
+  if (tpu && (cfg->cap & PIXMA_CAP_NEGATIVE))
     {
       ss->mode_list[i] = SANE_I18N ("Negative color");
       ss->mode_map[i] = PIXMA_SCAN_MODE_NEGATIVE_COLOR;
@@ -365,7 +385,13 @@ create_mode_list (pixma_sane_t * ss, SANE_Bool tpu)
           i++;
         }
     }
-  if (!tpu && cfg->cap & PIXMA_CAP_48BIT)
+  if (tpu && (cfg->cap & PIXMA_CAP_TPUIR) == PIXMA_CAP_TPUIR)
+    {
+      ss->mode_list[i] = SANE_I18N ("Infrared");
+      ss->mode_map[i] = PIXMA_SCAN_MODE_TPUIR;
+      i++;
+    }
+  if (!tpu && (cfg->cap & PIXMA_CAP_48BIT))
     {
       ss->mode_list[i] = SANE_I18N ("48 bits color");
       ss->mode_map[i] = PIXMA_SCAN_MODE_COLOR_48;
@@ -377,7 +403,7 @@ create_mode_list (pixma_sane_t * ss, SANE_Bool tpu)
           i++;
         }
     }
-  if (!tpu && cfg->cap & PIXMA_CAP_LINEART)
+  if (!tpu && (cfg->cap & PIXMA_CAP_LINEART))
     {
       ss->mode_list[i] = SANE_VALUE_SCAN_MODE_LINEART;
       ss->mode_map[i] = PIXMA_SCAN_MODE_LINEART;
@@ -389,56 +415,70 @@ create_mode_list (pixma_sane_t * ss, SANE_Bool tpu)
 }
 
 /* create dynamic dpi_list
- * ss:      scanner device
- * ext = 0: min = 75 dpi; max = cfg->xdpi
- * ext = 1: use settings for ADF/TPU
- *          overrides hires
- *          cfg->ext_min_dpi and cfg->ext_max_dpi not set: min = 75 dpi; max = cfg->xdpi
- *          only cfg->ext_min_dpi set: min = cfg->ext_min_dpi; max = cfg->xdpi
- *          only cfg->ext_max_dpi set: min = 75 dpi; max = cfg->ext_max_dpi
- *          both cfg->ext_min_dpi and cfg->ext_max_dpi set: min = cfg->ext_min_dpi; cfg->ext_max_dpi
- * hires = 0: normal usage
- * hires = 1: min = 150 dpi; max = use calculation from ext */
+ * ss: scanner device */
 static void
-create_dpi_list (pixma_sane_t * ss, SANE_Bool ext, SANE_Bool hires)
+create_dpi_list (pixma_sane_t * ss)
 {
   const pixma_config_t *cfg;
   int i, j;
-  int min_dpi;
+  int min;
+  unsigned min_dpi;
+  unsigned max_dpi;
 
   cfg = pixma_get_config (ss->s);
 
+  /* get min/max dpi */
+  max_dpi = cfg->xdpi;
+  min_dpi = 75;
+  if (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_TPU
+      && ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_TPUIR)
+  { /* IR mode */
+    /*PDBG (pixma_dbg (4, "*create_dpi_list***** TPUIR mode\n"));*/
+    min_dpi = (cfg->tpuir_min_dpi) ? cfg->tpuir_min_dpi : 75;
+    max_dpi = (cfg->tpuir_max_dpi) ? cfg->tpuir_max_dpi : cfg->xdpi;
+  }
+  else if (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_TPU
+            || ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_ADF
+            || ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_ADFDUP)
+  { /* ADF / TPU mode */
+    /*PDBG (pixma_dbg (4, "*create_dpi_list***** ADF/TPU mode\n"));*/
+    min_dpi = (cfg->adftpu_min_dpi) ? cfg->adftpu_min_dpi : 75;
+    max_dpi = (cfg->adftpu_max_dpi) ? cfg->adftpu_max_dpi : cfg->xdpi;
+  }
+  else if (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_FLATBED
+            && (ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_COLOR_48
+                || ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_GRAY_16))
+  { /* 48 bits flatbed */
+    /*PDBG (pixma_dbg (4, "*create_dpi_list***** 48 bits flatbed mode\n"));*/
+    min_dpi = 150;
+  }
+
   /* set j for min. dpi
    *  75 dpi: j = 0
    * 150 dpi: j = 1 \
-   * 300 dpi: j = 2 |--> from cfg->ext_min_dpi for ADF/TPU
-   * ...            /
+   * 300 dpi: j = 2 |--> from cfg->adftpu_min_dpi or cfg->tpuir_min_dpi
+   * 600 dpi: j = 3 /
    * */
-  j = (hires ? 1 : 0);
-  if (ext && cfg->ext_min_dpi)
-    {
-      j = -1;
-      min_dpi = cfg->ext_min_dpi / 75;
-      do
-        {
-          j++;
-          min_dpi >>= 1;
-        }
-      while (min_dpi > 0);
-    }
+  j = -1;
+  min = min_dpi / 75;
+  do
+  {
+    j++;
+    min >>= 1;
+  }
+  while (min > 0);
 
   /* create dpi_list
-   * use j for min. dpi
-   * max. dpi is cfg->xdpi or cfg->ext_max_dpi for ADF/TPU */
+   * use j for min. dpi */
   i = 0;
   do
     {
       i++; j++;
       ss->dpi_list[i] = 75 * (1 << (j - 1));    /* 75 x 2^(j-1) */
     }
-  while ((unsigned) ss->dpi_list[i] != ((ext && cfg->ext_max_dpi) ? cfg->ext_max_dpi
-                                                                  : cfg->xdpi));
+  while ((unsigned) ss->dpi_list[i] < max_dpi);
   ss->dpi_list[0] = i;
+  /*PDBG (pixma_dbg (4, "*create_dpi_list***** min_dpi = %d, max_dpi = %d\n", min_dpi, max_dpi));*/
 }
 
 static void
@@ -544,7 +584,6 @@ control_string_option (pixma_sane_t * ss, SANE_Int n, SANE_Action a, void *v,
   option_descriptor_t *opt = &(OPT_IN_CTX[n]);
   const SANE_String_Const *slist = opt->sod.constraint.string_list;
   SANE_String str = (SANE_String) v;
-  int i;
 
   if (opt->sod.constraint_type == SANE_CONSTRAINT_NONE)
     {
@@ -565,6 +604,8 @@ control_string_option (pixma_sane_t * ss, SANE_Int n, SANE_Action a, void *v,
     }
   else
     {
+      int i;
+
       switch (a)
         {
         case SANE_ACTION_GET_VALUE:
@@ -598,11 +639,17 @@ control_option (pixma_sane_t * ss, SANE_Int n,
 {
   int result, i;
   const pixma_config_t *cfg;
+  SANE_Int dummy;
 
+  /* info may be null, better to set a dummy here then test everywhere */
+  if (info == NULL) 
+    info = &dummy;
+  
   cfg = pixma_get_config (ss->s);
 
   /* PDBG (pixma_dbg (4, "*control_option***** n = %u, a = %u\n", n, a)); */
 
+  /* first deal with options that require special treatment */
   result = SANE_STATUS_UNSUPPORTED;
   switch (n)
     {
@@ -638,8 +685,19 @@ control_option (pixma_sane_t * ss, SANE_Int n,
             return SANE_STATUS_INVAL;
           }
         break;
+      case opt_button_1:
+      case opt_button_2:
+      case opt_original:
+      case opt_target:
+      case opt_scan_resolution:
+        /* poll scanner if option is not cached */
+        if (! ss->button_option_is_cached[ BUTTON_GROUP_INDEX(n) ] )
+          update_button_state (ss, info);
+        /* mark this option as read */
+        ss->button_option_is_cached[  BUTTON_GROUP_INDEX(n) ] = 0;
     }
 
+  /* now deal with getting and setting of options */ 
   switch (SOD (n).type)
     {
     case SANE_TYPE_BOOL:
@@ -659,6 +717,7 @@ control_option (pixma_sane_t * ss, SANE_Int n,
   if (result != SANE_STATUS_GOOD)
     return result;
 
+  /* deal with dependencies between options */
   switch (n)
     {
     case opt_custom_gamma:
@@ -678,18 +737,13 @@ control_option (pixma_sane_t * ss, SANE_Int n,
         }
       break;
     case opt_mode:
-      if (cfg->cap & (PIXMA_CAP_LINEART)
+      if (cfg->cap & (PIXMA_CAP_48BIT|PIXMA_CAP_LINEART|PIXMA_CAP_TPUIR)
           && (a == SANE_ACTION_SET_VALUE || a == SANE_ACTION_SET_AUTO))
         { /* new mode selected: Color, Gray, ... */
           /* PDBG (pixma_dbg (4, "*control_option***** mode = %u *\n",
                            ss->mode_map[OVAL (opt_mode).w])); */
-          if (ss->source_map[OVAL (opt_source).w] != PIXMA_SOURCE_TPU)
-            { /* ADF or flatbed*/
-              /* recreate dynamic dpi_list */
-              create_dpi_list (ss, SANE_FALSE,
-                               (ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_COLOR_48
-                                || ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_GRAY_16));
-            }
+          /* recreate dynamic lists */
+          create_dpi_list (ss);
           if (ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_LINEART)
             { /* lineart */
               enable_option (ss, opt_threshold, SANE_TRUE);
@@ -704,28 +758,20 @@ control_option (pixma_sane_t * ss, SANE_Int n,
         }
       break;
     case opt_source:
-      if (cfg->cap & (PIXMA_CAP_ADF|PIXMA_CAP_ADFDUP|PIXMA_CAP_TPU)
+      if ((cfg->cap & (PIXMA_CAP_ADF|PIXMA_CAP_ADFDUP|PIXMA_CAP_TPU))
           && (a == SANE_ACTION_SET_VALUE || a == SANE_ACTION_SET_AUTO))
         { /* new source selected: flatbed, ADF, TPU, ... */
+          /* to avoid fatal errors,
+           * select first entry of dynamic mode_list
+           * identifiers are unknown here */
+          OVAL (opt_mode).w = ss->mode_map[0];
           /* recreate dynamic lists */
-          if (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_TPU)
-            { /* TPU mode */
-              create_mode_list (ss, SANE_TRUE);
-              create_dpi_list (ss, SANE_TRUE, SANE_FALSE);
-            }
-          else if (ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_ADF
-                   || ss->source_map[OVAL (opt_source).w] == PIXMA_SOURCE_ADFDUP)
-            { /* ADF mode */
-              create_mode_list (ss, SANE_FALSE);
-              create_dpi_list (ss, SANE_TRUE, SANE_FALSE);
-            }
-          else
-            { /* flatbed mode */
-              create_mode_list (ss, SANE_FALSE);
-              create_dpi_list (ss, SANE_FALSE,
-                               (ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_COLOR_48
-                                || ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_GRAY_16));
-            }
+          create_mode_list (ss);
+          create_dpi_list (ss);
+          /* to avoid fatal errors,
+           * select first entry of dynamic dpi_list
+           * identifiers are unknown here */
+          OVAL (opt_resolution).w = ss->dpi_list[1];
           if (ss->mode_map[OVAL (opt_mode).w] == PIXMA_SCAN_MODE_LINEART)
             { /* lineart */
               enable_option (ss, opt_threshold, SANE_TRUE);
@@ -736,12 +782,6 @@ control_option (pixma_sane_t * ss, SANE_Int n,
               enable_option (ss, opt_threshold, SANE_FALSE);
               enable_option (ss, opt_threshold_curve, SANE_FALSE);
             }
-          /* to avoid fatal errors,
-           * select first entry of dynamic dpi_list
-           * and first entry of dynamic mode_list
-           * identifiers are unknown here */
-          OVAL (opt_resolution).w = ss->dpi_list[1];
-          OVAL (opt_mode).w = ss->mode_map[0];
           *info |= SANE_INFO_RELOAD_OPTIONS;
         }
       break;
@@ -837,14 +877,14 @@ init_option_descriptors (pixma_sane_t * ss)
   ss->yrange.max = SANE_FIX (cfg->height / 75.0 * 25.4);
   ss->yrange.quant = SANE_FIX (0);
 
-  /* setup dpi up to the value supported by the scanner. */
-  create_dpi_list (ss, SANE_FALSE, SANE_FALSE);
-
   /* mode_list and source_list were already NULL-terminated,
    * because the whole pixma_sane_t was cleared during allocation. */
 
   /* setup available mode. */
-  create_mode_list (ss, SANE_FALSE);
+  create_mode_list (ss);
+
+  /* setup dpi up to the value supported by the scanner. */
+  create_dpi_list (ss);
 
   /* setup paper source */
   i = 0;
@@ -948,7 +988,7 @@ reader_loop (pixma_sane_t * ss)
 #ifndef NDEBUG
       pixma_dbg (1, "==== Button-controlled scan mode is enabled.\n");
       pixma_dbg (1, "==== To proceed, press 'SCAN' or 'COLOR' button. "
-		 "To cancel, press 'GRAY' button.\n");
+		 "To cancel, press 'GRAY' or 'END' button.\n");
 #endif
       while (pixma_wait_event (ss->s, 10) != 0)
         {
@@ -1271,7 +1311,7 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 SANE_Status
 sane_open (SANE_String_Const name, SANE_Handle * h)
 {
-  unsigned i, nscanners;
+  unsigned i, j, nscanners;
   int error = 0;
   pixma_sane_t *ss = NULL;
   const pixma_config_t *cfg;
@@ -1331,7 +1371,8 @@ sane_open (SANE_String_Const name, SANE_Handle * h)
   ss->rpipe = -1;
   ss->idle = SANE_TRUE;
   ss->scanning = SANE_FALSE;
-
+  for (j=0; j < BUTTON_GROUP_SIZE; j++)
+    ss->button_option_is_cached[j] = 0;
   error = pixma_open (i, &ss->s);
   if (error < 0)
     {
@@ -1638,7 +1679,7 @@ type int resolution
 
 type string mode[30]
   constraint @string_list = ss->mode_list
-  default @s = SANE_I18N(ss->mode_list[0])
+  default @s = SANE_VALUE_SCAN_MODE_COLOR
   title @SANE_TITLE_SCAN_MODE
   desc  @SANE_DESC_SCAN_MODE
   cap soft_select soft_detect automatic
@@ -1731,12 +1772,27 @@ type button button-update
 type int button-1
   default 0
   title Button 1
-  cap soft_select soft_detect advanced
+  cap soft_detect advanced
 
 type int button-2
   default 0
   title Button 2
-  cap soft_select soft_detect advanced
+  cap soft_detect advanced
+
+type int original
+  default 0
+  title Type of original to scan
+  cap soft_detect advanced
+
+type int target
+  default 0
+  title Target operation type
+  cap soft_detect advanced 
+
+type int scan-resolution
+  default 0
+  title Scan resolution
+  cap soft_detect advanced
 
 rem -------------------------------------------
 type group
@@ -1762,5 +1818,8 @@ END SANE_Option_Descriptor
 
 /* pixma_sane_options.c generated by
  * scripts/pixma_gen_options.py < pixma.c > pixma_sane_options.c
- * *and* formatted */
+ *
+ * pixma_sane_options.h generated by
+ * scripts/pixma_gen_options.py h < pixma.c > pixma_sane_options.h
+ */
 #include "pixma_sane_options.c"
