@@ -1,15 +1,10 @@
-/*.............................................................................
- * Project : SANE library for Plustek parallelport flatbed scanners.
- *.............................................................................
- */
-
 /* @file plustekpp-io.c
  * @brief as the name says, here we have all the I/O
  *        functions according to the parallel port hardware
  *
  * based on sources acquired from Plustek Inc.
  * Copyright (C) 1998 Plustek Inc.
- * Copyright (C) 2000-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2000-2013 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.37 - initial version
@@ -24,6 +19,9 @@
  * - 0.40 - no changes
  * - 0.41 - no changes
  * - 0.42 - changed include names
+ * - 0.43 - no changes
+ * - 0.44 - fix format string issues, as Long types default to int32_t
+ *          now
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -381,7 +379,7 @@ static void ioRestoreParallelMode( pScanData ps )
 
 /** try to connect to scanner (ASIC 9600x and 98001)
  */
-_LOC _INL void ioP98001EstablishScannerConnection( pScanData ps, ULong delTime )
+_LOC void ioP98001EstablishScannerConnection( pScanData ps, ULong delTime )
 {
 	_OUTB_DATA( ps, _ID_TO_PRINTER );
     _DO_UDELAY( delTime );
@@ -556,7 +554,7 @@ static int ioP98ReadWriteTest( pScanData ps )
 	IODataToRegister( ps, ps->RegMemoryHigh, 0 );
 
 	/* fill to buffer */
-	IOMoveDataToScanner( ps, buffer, 1280 );		
+	IOMoveDataToScanner( ps, buffer, _MEMTEST_SIZE );
 
 	IODataToRegister( ps, ps->RegModeControl, _ModeMappingMem );
 	IODataToRegister( ps, ps->RegMemoryLow,  0 );
@@ -579,7 +577,7 @@ static int ioP98ReadWriteTest( pScanData ps )
 
 	for( ul = 0; ul < _MEMTEST_SIZE; ul++ ) {
 		if( buffer[ul] != buffer[ul+_MEMTEST_SIZE] ) {
-			DBG( DBG_HIGH, "Error in memory test at pos %lu (%u != %u)\n",
+			DBG( DBG_HIGH, "Error in memory test at pos %u (%u != %u)\n",
 				 ul, buffer[ul], buffer[ul+_MEMTEST_SIZE] );
 			retval = _E_NO_DEV;
 			break;
@@ -594,7 +592,7 @@ static int ioP98ReadWriteTest( pScanData ps )
  */
 static void ioSPPWrite( pScanData ps, pUChar pBuffer, ULong size )
 {
-	DBG( DBG_IO , "Moving %lu bytes to scanner, IODELAY = %u...\n",
+	DBG( DBG_IO , "Moving %u bytes to scanner, IODELAY = %u...\n",
 					size, ps->IO.delay );
 	switch( ps->IO.delay ) {
 	
