@@ -1,4 +1,4 @@
-static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.17 2008-11-26 21:21:30 kitno-guest Exp $";
+static const char RCSid[] = "$Header$";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -44,9 +44,9 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
    Written by Randolph Bentson <bentson@holmsjoen.com> */
 
 /*
- * $Log: sp15c.c,v $
- * Revision 1.17  2008-11-26 21:21:30  kitno-guest
- * * backend/*.[ch]: nearly every backend used V_MAJOR
+ * $Log$
+ * Revision 1.17  2008/11/26 21:21:30  kitno-guest
+ * * backend/ *.[ch]: nearly every backend used V_MAJOR
  * instead of SANE_CURRENT_MAJOR in sane_init()
  * * backend/snapscan.c: remove EXPECTED_VERSION check
  * since new SANE standard is forward compatible
@@ -122,9 +122,9 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
  * 	  backend/qcam.c backend/ricoh.c backend/s9036.c backend/snapscan.c
  * 	  backend/tamarack.c: Use sanei_config_read instead of fgets.
  * 	* backend/dc210.c backend/microtek.c backend/pnm.c: Added
- * 	  #include <sane/config.h>.
+ * 	  #include "../include/sane/config.h".
  * 	* backend/dc25.c backend/m3096.c  backend/sp15.c
- *  	  backend/st400.c: Moved #include <sane/config.h> to the beginning.
+ *  	  backend/st400.c: Moved #include "../include/sane/config.h" to the beginning.
  * 	* AUTHORS: Changed agfa to agfafocus.
  *
  * Revision 1.1.2.3  2000/03/14 17:47:12  abel
@@ -207,7 +207,7 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 
 /* ------------------------------------------------------------------------- */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -220,11 +220,11 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "sane/sanei_backend.h"
-#include "sane/sanei_scsi.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_config.h"
-#include "sane/sanei_thread.h"
+#include "../include/sane/sanei_backend.h"
+#include "../include/sane/sanei_scsi.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/sanei_thread.h"
 
 #include "sp15c-scsi.h"
 #include "sp15c.h"
@@ -248,11 +248,11 @@ static const SANE_Int x_res_list[] =
 static const SANE_Int y_res_list[] =
 {11, 0, 60, 75, 80, 100, 120, 150, 200, 240, 300, 600};
 
-static const char lineStr[] = "Lineart";
-static const char halfStr[] = "Halftone";
+static const char lineStr[] = SANE_VALUE_SCAN_MODE_LINEART;
+static const char halfStr[] = SANE_VALUE_SCAN_MODE_HALFTONE;
 static const char gray4Str[] = "4-bit Gray";
 static const char gray8Str[] = "8-bit Gray";
-static const char colorStr[] = "Color";
+static const char colorStr[] = SANE_VALUE_SCAN_MODE_COLOR;
 static SANE_String_Const scan_mode_list[] =
 {lineStr, halfStr, gray4Str, gray8Str, colorStr, NULL};
 
@@ -311,7 +311,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   struct sp15c *dev;
   int i;
   
@@ -990,6 +989,9 @@ sane_exit (void)
       free (dev->buffer);
       free (dev);
     }
+  
+  if (devlist)
+    free (devlist);
 }                               /* sane_exit */
 
 /* }################ internal (support) routines ################{ */
