@@ -113,8 +113,9 @@ typedef struct Tamarack_Scanner
     int resolution_code;
 #endif
     int fd;			/* SCSI filedescriptor */
-    pid_t reader_pid;		/* process id of reader */
+    SANE_Pid reader_pid;	/* process id of reader */
     int pipe;			/* pipe to reader process */
+    int reader_pipe;		/* pipe from reader process */
 
     /* scanner dependent/low-level state: */
     Tamarack_Device *hw;
