@@ -80,7 +80,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#define BACKEND_VERSION "0.02-10"
+#define BACKEND_VERSION "0.02-11"
 #define BACKEND_NAME    u12
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/sanei_config.h"
@@ -385,13 +385,13 @@ static int reader_process( void *args )
 static SANE_Status do_cancel( U12_Scanner *scanner, SANE_Bool closepipe )
 {
 	struct SIGACTION act;
-	pid_t            res;
+	SANE_Pid         res;
 
 	DBG( _DBG_PROC,"do_cancel\n" );
 
 	scanner->scanning = SANE_FALSE;
 
-	if( scanner->reader_pid > 0 ) {
+	if( scanner->reader_pid != -1 ) {
 
 		DBG( _DBG_PROC, ">>>>>>>> killing reader_process <<<<<<<<\n" );
 
@@ -421,7 +421,7 @@ static SANE_Status do_cancel( U12_Scanner *scanner, SANE_Bool closepipe )
 			sanei_thread_sendsig( scanner->reader_pid, SIGKILL );
 #endif
 		}
-		scanner->reader_pid = 0;
+		scanner->reader_pid = -1;
 		DBG( _DBG_PROC, "reader_process killed\n");
 
 		if( scanner->hw->fd >= 0 ) {
@@ -848,8 +848,8 @@ static SANE_Status attach( const char *dev_name,
 	dev->sane.name   = dev->name;           /* compiler warnings        */
 	dev->sane.vendor = "Plustek";
 	dev->sane.model  = "U12/1212U";
-	dev->sane.type   = "USB flatbed scanner";
-	dev->initialized = SANE_FALSE;         
+	dev->sane.type   = SANE_I18N ("flatbed scanner");
+	dev->initialized = SANE_FALSE;
 
 	memcpy( &dev->adj, &cnf->adj, sizeof(AdjDef));
 	show_cnf( cnf );
@@ -943,7 +943,7 @@ SANE_Status sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 	init_config_struct( &config );
 
 	if( version_code != NULL )
-		*version_code = SANE_VERSION_CODE(V_MAJOR, V_MINOR, 0);
+		*version_code = SANE_VERSION_CODE(SANE_CURRENT_MAJOR, V_MINOR, 0);
 
 	fp = sanei_config_open( U12_CONFIG_FILE );
 
@@ -1731,9 +1731,9 @@ SANE_Status sane_start( SANE_Handle handle )
 	
 	cancelRead = SANE_FALSE;
 	
-	if( s->reader_pid < 0 ) {
+	if( s->reader_pid == -1 ) {
 		DBG( _DBG_ERROR, "ERROR: could not start reader task\n" );
-	    s->scanning = SANE_FALSE;
+		s->scanning = SANE_FALSE;
 		u12if_close( dev );
 		return SANE_STATUS_IO_ERROR;
 	}
