@@ -41,7 +41,7 @@
    This file implements the Mustek-proprietary SCSI-over-parallel-port
    interface.  */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -51,6 +51,9 @@
 
 #ifdef HAVE_SYS_IO_H
 # include <sys/io.h>	/* use where available (glibc 2.x, for example) */
+# ifndef SANE_HAVE_SYS_IO_H_WITH_INB_OUTB
+#  define IO_SUPPORT_MISSING
+# endif
 #elif HAVE_ASM_IO_H
 # include <asm/io.h>	/* ugly, but backwards compatible */
 #elif defined (__i386__)  && defined (__GNUC__)
@@ -74,9 +77,9 @@ inb (u_long port)
 # define IO_SUPPORT_MISSING
 #endif
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/sanei_ab306.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/sanei_ab306.h"
 
 #if (defined(HAVE_IOPERM) || defined(__FreeBSD__)) && !defined(IO_SUPPORT_MISSING)
 
@@ -91,10 +94,10 @@ inb (u_long port)
 #include <sys/wait.h>
 #include <unistd.h>
 
-#include "sane/saneopts.h"
+#include "../include/sane/saneopts.h"
 
 #define BACKEND_NAME sanei_ab306
-#include "sane/sanei_debug.h"
+#include "../include/sane/sanei_debug.h"
 
 #define PORT_DEV	"/dev/port"
 #define AB306_CIO	0x379		/* control i/o port */
