@@ -1,5 +1,6 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 2000-2003 Jochen Eisinger <jochen.eisinger@gmx.net>
+   Copyright (C) 2003 James Perry (scsi_pp functions)
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -52,10 +53,10 @@
    These debug levels can be set using the environment variable
    SANE_DEBUG_SANEI_PA4S2 */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #define BACKEND_NAME sanei_pa4s2
-#include "sane/sanei_backend.h"	/* pick up compatibility defs */
+#include "../include/sane/sanei_backend.h"	/* pick up compatibility defs */
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -71,7 +72,10 @@
 #  if defined (__ICC) && __ICC >= 700
 #   define __GNUC__ 2
 #  endif
-# include <sys/io.h>
+#  include <sys/io.h>
+#  ifndef SANE_HAVE_SYS_IO_H_WITH_INB_OUTB
+#   define IO_SUPPORT_MISSING
+#  endif
 #  if defined (__ICC) && __ICC >= 700
 #   undef __GNUC__
 #  elif defined(__ICC) && defined(HAVE_ASM_IO_H)
@@ -108,9 +112,9 @@ inb (u_long port)
 
 #endif /* HAVE_LIBIEEE1284 */
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/sanei_pa4s2.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/sanei_pa4s2.h"
 
 
 #ifdef NDEBUG
@@ -160,7 +164,7 @@ static int sanei_pa4s2_dbg_init_called = SANE_FALSE;
 # include <sys/types.h>
 #endif
 
-#include "sane/saneopts.h"
+#include "../include/sane/saneopts.h"
 
 
 #if (defined (HAVE_IOPERM) || defined (HAVE_LIBIEEE1284)) && !defined (IO_SUPPORT_MISSING)
@@ -179,7 +183,7 @@ static int sanei_pa4s2_dbg_init_called = SANE_FALSE;
 # include <sys/types.h>
 #endif
 
-#include "sane/saneopts.h"
+#include "../include/sane/saneopts.h"
 
 #define PA4S2_MODE_NIB	0
 #define PA4S2_MODE_UNI	1
@@ -555,9 +559,9 @@ pa4s2_open (const char *dev, SANE_Status * status)
 #define inbyte2(fd)	(ieee1284_read_control(pplist.portv[fd]) ^ C1284_INVERTED)
 static u_char inbyte4(int fd)
 {
-  u_char val;
+  char val;
   ieee1284_epp_read_data(pplist.portv[fd], 0, &val, 1);
-  return val;
+  return (u_char)val;
 }
 
 #define outbyte0(fd,val)	ieee1284_write_data(pplist.portv[fd], val)
@@ -566,7 +570,7 @@ static u_char inbyte4(int fd)
 
 static void outbyte3(int fd, u_char val)
 {
-  ieee1284_epp_write_addr (pplist.portv[fd], 0, &val, 1);
+  ieee1284_epp_write_addr (pplist.portv[fd], 0, (char *)&val, 1);
 }
 
 #else
@@ -911,13 +915,15 @@ pa4s2_close (int fd, SANE_Status * status)
 
 }
 
-char **
+const char **
 sanei_pa4s2_devices()
 {
 
   SANE_Status status;
   int n;
-  char **devices;
+  const char **devices;
+
+  TEST_DBG_INIT();
 
   DBG (4, "sanei_pa4s2_devices: invoked\n");
 
@@ -937,7 +943,9 @@ sanei_pa4s2_devices()
       return calloc(1, sizeof(char *));
     }
 
-  memcpy(devices, pplist.portv, sizeof(char *) * pplist.portc);
+  for (n=0; n<pplist.portc; n++)
+    devices[n] = pplist.portv[n]->name;
+
 #else
 
   if ((devices = calloc((NELEMS (port) + 1), sizeof(char *))) == NULL)
@@ -953,6 +961,217 @@ sanei_pa4s2_devices()
   return devices;
 }
 
+/*
+ * Needed for SCSI-over-parallel scanners (Paragon 600 II EP)
+ */
+SANE_Status
+sanei_pa4s2_scsi_pp_get_status(int fd, u_char *status)
+{
+  u_char stat;
+
+  TEST_DBG_INIT ();
+
+  DBG (6, "sanei_pa4s2_scsi_pp_get_status: called for fd %d\n",
+       fd);
+
+#if defined(HAVE_LIBIEEE1284)
+  if ((fd < 0) || (fd >= pplist.portc))
+#else
+  if ((fd < 0) || (fd >= NELEMS (port)))
+#endif
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_get_status: invalid fd %d\n", fd);
+      DBG (6, "sanei_pa4s2_scsi_pp_get_status: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+  if (port[fd].in_use == SANE_FALSE)
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_get_status: port is not in use\n");
+#if defined(HAVE_LIBIEEE1284)
+      DBG (4, "sanei_pa4s2_scsi_pp_get_status: port is '%s'\n",
+      		pplist.portv[fd]->name);
+#else
+      DBG (6, "sanei_pa4s2_scsi_pp_get_status: port is 0x%03lx\n",
+	   port[fd].base);
+#endif
+      DBG (5, "sanei_pa4s2_scsi_pp_get_status: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+  if (port[fd].enabled == SANE_FALSE)
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_get_status: port is not enabled\n");
+#if defined(HAVE_LIBIEEE1284)
+      DBG (4, "sanei_pa4s2_scsi_pp_get_status: port is '%s'\n",
+      		pplist.portv[fd]->name);
+#else
+      DBG (6, "sanei_pa4s2_scsi_pp_get_status: port is 0x%03lx\n",
+	   port[fd].base);
+#endif
+      DBG (5, "sanei_pa4s2_scsi_pp_get_status: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+  outbyte2 (fd, 0x4);
+  stat = inbyte1 (fd)^0x80;
+  *status = (stat&0x2f)|((stat&0x10)<<2)|((stat&0x40)<<1)|((stat&0x80)>>3);
+  DBG (5, "sanei_pa4s2_scsi_pp_get_status: status=0x%02X\n", *status);
+  DBG (6, "sanei_pa4s2_scsi_pp_get_status: returning SANE_STATUS_GOOD\n");
+
+  return SANE_STATUS_GOOD;
+}
+
+/*
+ * SCSI-over-parallel scanners need this done when a register is
+ * selected
+ */ 
+SANE_Status
+sanei_pa4s2_scsi_pp_reg_select (int fd, int reg)
+{
+  TEST_DBG_INIT ();
+
+#if defined(HAVE_LIBIEEE1284)
+  if ((fd < 0) || (fd >= pplist.portc))
+#else
+  if ((fd < 0) || (fd >= NELEMS (port)))
+#endif
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_reg_select: invalid fd %d\n", fd);
+      DBG (6, "sanei_pa4s2_scsi_pp_reg_select: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+  if (port[fd].in_use == SANE_FALSE)
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_reg_select: port is not in use\n");
+#if defined(HAVE_LIBIEEE1284)
+      DBG (4, "sanei_pa4s2_scsi_pp_get_status: port is '%s'\n",
+      		pplist.portv[fd]->name);
+#else
+      DBG (6, "sanei_pa4s2_scsi_pp_get_status: port is 0x%03lx\n",
+	   port[fd].base);
+#endif
+      DBG (5, "sanei_pa4s2_scsi_pp_reg_select: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+  if (port[fd].enabled == SANE_FALSE)
+    {
+
+      DBG (2, "sanei_pa4s2_scsi_pp_reg_select: port is not enabled\n");
+#if defined(HAVE_LIBIEEE1284)
+      DBG (4, "sanei_pa4s2_scsi_pp_get_status: port is '%s'\n",
+      		pplist.portv[fd]->name);
+#else
+      DBG (6, "sanei_pa4s2_scsi_pp_get_status: port is 0x%03lx\n",
+	   port[fd].base);
+#endif
+      DBG (5, "sanei_pa4s2_scsi_pp_reg_select: returning SANE_STATUS_INVAL\n");
+
+      return SANE_STATUS_INVAL;
+
+    }
+
+#if defined(HAVE_LIBIEEE1284)
+  DBG (6, "sanei_pa4s2_scsi_pp_reg_select: selecting register %u at port '%s'\n",
+       (int) reg, pplist.portv[fd]->name);
+#else
+  DBG (6, "sanei_pa4s2_scsi_pp_reg_select: selecting register %u at 0x%03lx\n",
+       (int) reg, (u_long)port[fd].base);
+#endif
+
+  outbyte0 (fd, reg | 0x58);
+  outbyte2 (fd, 0x04);
+  outbyte2 (fd, 0x06);
+  outbyte2 (fd, 0x04);
+  outbyte2 (fd, 0x04);
+
+  return SANE_STATUS_GOOD;
+}
+
+/*
+ * The SCSI-over-parallel scanners need to be handled a bit differently
+ * when opened, as they don't return a valid ASIC ID, so this can't be
+ * used for detecting valid read modes
+ */
+SANE_Status
+sanei_pa4s2_scsi_pp_open (const char *dev, int *fd)
+{
+
+  u_char val;
+  SANE_Status status;
+
+  TEST_DBG_INIT ();
+
+  DBG(4, "sanei_pa4s2_scsi_pp_open: called for device '%s'\n", dev);
+  DBG(5, "sanei_pa4s2_scsi_pp_open: trying to connect to port\n");
+
+  if ((*fd = pa4s2_open (dev, &status)) == -1)
+    {
+
+      DBG (5, "sanei_pa4s2_scsi_pp_open: connection failed\n");
+
+      return status;
+
+    }
+
+  DBG (6, "sanei_pa4s2_scsi_pp_open: connected to device using fd %u\n", *fd);
+
+  DBG (5, "sanei_pa4s2_scsi_pp_open: checking for scanner\n");
+
+  if (sanei_pa4s2_enable (*fd, SANE_TRUE)!=SANE_STATUS_GOOD)
+    {
+      DBG (3, "sanei_pa4s2_scsi_pp_open: error enabling device\n");
+      return SANE_STATUS_IO_ERROR;
+    }
+
+  /*
+   * Instead of checking ASIC ID, check device status
+   */
+  if (sanei_pa4s2_scsi_pp_get_status(*fd, &val)!=SANE_STATUS_GOOD)
+    {
+      DBG (3, "sanei_pa4s2_scsi_pp_open: error getting device status\n");
+      sanei_pa4s2_enable (*fd, SANE_FALSE);
+      return SANE_STATUS_IO_ERROR;
+    }
+  val&=0xf0;
+
+  if ((val==0xf0)||(val&0x40)||(!(val&0x20)))
+    {
+      DBG (3, "sanei_pa4s2_scsi_pp_open: device returned status 0x%02X\n", val);
+      sanei_pa4s2_enable (*fd, SANE_FALSE);
+      return SANE_STATUS_DEVICE_BUSY;
+    }
+
+  if (sanei_pa4s2_enable (*fd, SANE_FALSE)!=SANE_STATUS_GOOD)
+    {
+      DBG (3, "sanei_pa4s2_scsi_pp_open: error disabling device\n");
+      return SANE_STATUS_IO_ERROR;
+    }
+
+  /* FIXME: it would be nice to try to use a better mode here, but how to
+   * know if it's going to work? */
+
+  DBG (4, "sanei_pa4s2_scsi_pp_open: returning SANE_STATUS_GOOD\n");
+
+  return SANE_STATUS_GOOD;
+}
 
 SANE_Status
 sanei_pa4s2_open (const char *dev, int *fd)
@@ -1037,6 +1256,14 @@ sanei_pa4s2_open (const char *dev, int *fd)
 
 	}
 
+      if ((port[*fd].mode == PA4S2_MODE_EPP) &&
+      ((sanei_pa4s2_interface_options & SANEI_PA4S2_OPT_NO_EPP) != 0))
+	{
+	  DBG (3, "sanei_pa4s2_open: skipping mode EPP\n");
+	  break;
+	}
+
+
       DBG (5, "sanei_pa4s2_open: trying mode %u\n", port[*fd].mode);
 
       sanei_pa4s2_enable (*fd, SANE_TRUE);
@@ -1651,7 +1878,7 @@ sanei_pa4s2_options (u_int * options, int set)
   if ((set != SANE_TRUE) && (set != SANE_FALSE))
     DBG (2, "sanei_pa4s2_options: value of set is invalid\n");
 
-  if ((set == SANE_TRUE) && (*options > 3))
+  if ((set == SANE_TRUE) && (*options > 7))
     DBG (2, "sanei_pa4s2_options: value of *options is invalid\n");
 
   if (set == SANE_TRUE)
@@ -1834,7 +2061,7 @@ sanei_pa4s2_options (u_int * options, int set)
 
 }
 
-char **
+const char **
 sanei_pa4s2_devices()
 {
   TEST_DBG_INIT ();
@@ -1846,4 +2073,34 @@ sanei_pa4s2_devices()
   return calloc(1, sizeof(char *));
 }
 
+SANE_Status
+sanei_pa4s2_scsi_pp_get_status(int fd, u_char *status)
+{
+  TEST_DBG_INIT ();
+  DBG (4, "sanei_pa4s2_scsi_pp_get_status: fd=%d, status=%p\n",
+       fd, (void *) status);
+  DBG (3, "sanei_pa4s2_scsi_pp_get_status: A4S2 support not compiled\n");
+  return SANE_STATUS_UNSUPPORTED;
+}
+
+SANE_Status
+sanei_pa4s2_scsi_pp_reg_select (int fd, int reg)
+{
+  TEST_DBG_INIT ();
+  DBG (4, "sanei_pa4s2_scsi_pp_reg_select: fd=%d, reg=%d\n",
+       fd, reg);
+  DBG (3, "sanei_pa4s2_devices: A4S2 support not compiled\n");
+  return SANE_STATUS_UNSUPPORTED;
+}
+
+SANE_Status
+sanei_pa4s2_scsi_pp_open (const char *dev, int *fd)
+{
+  TEST_DBG_INIT ();
+  DBG (4, "sanei_pa4s2_scsi_pp_open: dev=%s, fd=%p\n",
+       dev, (void *) fd);
+  DBG (3, "sanei_pa4s2_scsi_pp_open: A4S2 support not compiled\n");
+  return SANE_STATUS_UNSUPPORTED;
+}
+
 #endif /* !HAVE_IOPERM */
